'''
–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
# Venn Diagram
Module | `venn_diagram.py`

Diagram that displays all possible logical relations between a finite\
collection of different resources.

Daniel Bakas Amuchastegui\
September 3, 2019

Copyright © Semantyk 2020. All rights reserved.\
–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
'''