# -*- coding: utf-8 -*-
##################################################
#
# test.py 
# Module | Test
#
# A file for testing the package.
#
# Author: Daniel Bakas Amuchastegui
# May 13, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
##################################################

import sys
import tkinter

def main():
    root = tkinter.Tk()
    root.title('Semantyk')
    root.resizable(width=False, height=False)
    root.mainloop()
if __name__ == '__main__': 
    main()