'''
–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
# Semantyk 
Package | `__init__.py`

Daniel Bakas Amuchastegui\
August 31, 2017

Copyright © Semantyk 2020. All rights reserved.\
–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
'''

__author__ = 'Daniel Bakas Amuchastegui'
__date__ = '2020-05-21'
__email__ = 'danielbakas@gmail.com'
__version__ = '0.0.0.60'

from . import (archive, av_reality, core, das, dots, dvm, games, learn, sam,
               store, tools, uuid_generator)
from .archive import *
from .av_reality import *
from .core import *
from .dots import *
from .dvm import *
from .games import *
from .learn import *
from .sam import *
from .store import *
from .tools import *
from .uuid_generator import *
