# -*- coding: utf-8 -*-
##################################################
#
# test_import.py 
# Module | Test Import
#
# A file for testing the import of the package.
#
# Author: Daniel Bakas Amuchastegui
# May 13, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
##################################################

import semantyk

def test_import():
    print('Semantyk ' + semantyk.__version__)

test_import()